@@ -13,6 +13,8 @@
 // limitations under the License.
 
 // +build e2e
+// +build !eventing
+// +build !serving
 
 package e2e
 
@@ -23,7 +25,7 @@ import (
 	"path/filepath"
 	"testing"
 
-	"gotest.tools/assert"
+	"gotest.tools/v3/assert"
 
 	"knative.dev/client/lib/test"
 	"knative.dev/client/pkg/util"
@@ -36,8 +38,8 @@ echo "Hello Knative, I'm a Kn plugin"
 echo "  My plugin file is $0"
 echo "  I received arguments: $1 $2 $3 $4"`
 
-	FileModeReadWrite  = 0666
-	FileModeExecutable = 0777
+	TestPluginCodeErr string = `#!/bin/bash
+exit 1`
 )
 
 type pluginTestConfig struct {
@@ -53,27 +55,27 @@ func (pc *pluginTestConfig) setup() error {
 	}
 
 	pc.knPluginsDir = filepath.Join(pc.knConfigDir, "plugins")
-	err = os.MkdirAll(pc.knPluginsDir, FileModeExecutable)
+	err = os.MkdirAll(pc.knPluginsDir, test.FileModeExecutable)
 	if err != nil {
 		return err
 	}
 
 	pc.knPluginsDir2 = filepath.Join(pc.knConfigDir, "plugins2")
-	err = os.MkdirAll(pc.knPluginsDir2, FileModeExecutable)
+	err = os.MkdirAll(pc.knPluginsDir2, test.FileModeExecutable)
 	if err != nil {
 		return err
 	}
 
-	pc.knConfigPath, err = createPluginFile("config.yaml", "", pc.knConfigDir, FileModeReadWrite)
+	pc.knConfigPath, err = test.CreateFile("config.yaml", "", pc.knConfigDir, test.FileModeReadWrite)
 	if err != nil {
 		return err
 	}
 
-	pc.knPluginPath, err = createPluginFile("kn-helloe2e", TestPluginCode, pc.knPluginsDir, FileModeExecutable)
+	pc.knPluginPath, err = test.CreateFile("kn-helloe2e", TestPluginCode, pc.knPluginsDir, test.FileModeExecutable)
 	if err != nil {
 		return err
 	}
-	pc.knPluginPath2, err = createPluginFile("kn-hello2e2e", TestPluginCode, pc.knPluginsDir2, FileModeExecutable)
+	pc.knPluginPath2, err = test.CreateFile("kn-hello2e2e", TestPluginCode, pc.knPluginsDir2, test.FileModeExecutable)
 	if err != nil {
 		return err
 	}
@@ -84,12 +86,6 @@ func (pc *pluginTestConfig) teardown() {
 	os.RemoveAll(pc.knConfigDir)
 }
 
-func createPluginFile(fileName, fileContent, filePath string, fileMode os.FileMode) (string, error) {
-	file := filepath.Join(filePath, fileName)
-	err := ioutil.WriteFile(file, []byte(fileContent), fileMode)
-	return file, err
-}
-
 func TestPluginWithoutLookup(t *testing.T) {
 	t.Parallel()
 
@@ -108,12 +104,24 @@ func TestPluginWithoutLookup(t *testing.T) {
 	listPlugin(r, knFlags, []string{pc.knPluginPath}, []string{})
 
 	t.Log("execute plugin in --plugins-dir")
-	runPlugin(r, knFlags, "helloe2e", []string{"e2e", "test"}, []string{"Hello Knative, I'm a Kn plugin", "I received arguments: e2e"})
+	runPlugin(r, knFlags, "helloe2e", []string{"e2e", "test"}, []string{"Hello Knative, I'm a Kn plugin", "I received arguments", "e2e"})
 
 	t.Log("does not list any other plugin in $PATH")
 	listPlugin(r, knFlags, []string{pc.knPluginPath}, []string{pc.knPluginPath2})
+}
 
-	t.Log("with --lookup-plugins is true")
+func TestPluginInHelpMessage(t *testing.T) {
+	pc := pluginTestConfig{}
+	assert.NilError(t, pc.setup())
+	defer pc.teardown()
+
+	result := test.Kn{}.Run("--plugins-dir", pc.knPluginsDir, "--help")
+	assert.NilError(t, result.Error)
+	assert.Assert(t, util.ContainsAll(result.Stdout, "Plugins:", "helloe2e", "kn-helloe2e"))
+
+	result = test.Kn{}.Run("--plugins-dir", pc.knPluginsDir, "service", "--help")
+	assert.NilError(t, result.Error)
+	assert.Assert(t, util.ContainsNone(result.Stdout, "Plugins:", "helloe2e", "kn-helloe2e"))
 }
 
 func TestPluginWithLookup(t *testing.T) {
@@ -167,6 +175,33 @@ func TestExecutePluginInPath(t *testing.T) {
 	runPlugin(r, knFlags, "hello2e2e", []string{}, []string{"Hello Knative, I'm a Kn plugin"})
 }
 
+func TestExecutePluginInPathWithError(t *testing.T) {
+	it, err := test.NewKnTest()
+	assert.NilError(t, err)
+
+	r := test.NewKnRunResultCollector(t, it)
+	defer r.DumpIfFailed()
+
+	pc := pluginTestConfig{}
+	assert.NilError(t, pc.setup())
+	oldPath := os.Getenv("PATH")
+
+	t.Log("execute plugin in $PATH that returns error")
+	pluginsDir := filepath.Join(pc.knConfigDir, "plugins3")
+	err = os.MkdirAll(pluginsDir, test.FileModeExecutable)
+	assert.NilError(t, err)
+	_, err = test.CreateFile("kn-hello3e2e", TestPluginCodeErr, pluginsDir, test.FileModeExecutable)
+	assert.NilError(t, err)
+	assert.NilError(t, os.Setenv("PATH", fmt.Sprintf("%s:%s", oldPath, pluginsDir)))
+	defer tearDownWithPath(pc, oldPath)
+
+	out := test.Kn{}.Run("--lookup-plugins=true", "hello3e2e")
+	r.AssertError(out)
+	assert.Check(r.T(), util.ContainsAll(out.Stderr, "Error: exit status 1"))
+	assert.Check(r.T(), util.ContainsNone(out.Stderr, "Run", "kn --help", "usage"))
+}
+
+// Private
 func setupPluginTestConfigWithNewPath(t *testing.T) (pluginTestConfig, string) {
 	pc := pluginTestConfig{}
 	assert.NilError(t, pc.setup())
@@ -180,15 +215,17 @@ func tearDownWithPath(pc pluginTestConfig, oldPath string) {
 	pc.teardown()
 }
 
-// Private
-
 func listPlugin(r *test.KnRunResultCollector, knFlags []string, expectedPlugins []string, unexpectedPlugins []string) {
 	knArgs := append(knFlags, "plugin", "list")
 
 	out := test.Kn{}.Run(knArgs...)
 	r.AssertNoError(out)
-	assert.Check(r.T(), util.ContainsAll(out.Stdout, expectedPlugins...))
-	assert.Check(r.T(), util.ContainsNone(out.Stdout, unexpectedPlugins...))
+	for _, p := range expectedPlugins {
+		assert.Check(r.T(), util.ContainsAll(out.Stdout, filepath.Base(p)))
+	}
+	for _, p := range unexpectedPlugins {
+		assert.Check(r.T(), util.ContainsNone(out.Stdout, filepath.Base(p)))
+	}
 }
 
 func runPlugin(r *test.KnRunResultCollector, knFlags []string, pluginName string, args []string, expectedOutput []string) {
